@@ -11,6 +11,7 @@ from autots.datasets import (
     load_weekly,
     load_weekdays,
     load_live_daily,
+    load_linear,
 )
 
 from autots.evaluator.auto_ts import AutoTS
@@ -19,7 +20,7 @@ from autots.tools.shaping import long_to_wide
 from autots.tools.regressor import create_lagged_regressor
 from autots.evaluator.auto_model import model_forecast
 
-__version__ = '0.3.6'
+__version__ = '0.3.7'
 
 TransformTS = GeneralTransformer
 
@@ -31,6 +32,7 @@ __all__ = [
     'load_weekly',
     'load_weekdays',
     'load_live_daily',
+    'load_linear',
     'AutoTS',
     'TransformTS',
     'GeneralTransformer',
