@@ -391,3 +391,47 @@ def load_zeroes(long=False, shape=None, start_date: str = "2021-01-01"):
             id_vars=['datetime'], var_name='series_id', value_name='value'
         )
         return df_long
+
+
+def load_linear(long=False, shape=None, start_date: str = "2021-01-01"):
+    """Create a dataset of just zeroes for testing edge case."""
+    if shape is None:
+        shape = (500, 5)
+    df_wide = pd.DataFrame(
+        np.ones(shape), index=pd.date_range(start_date, periods=shape[0], freq="D")
+    )
+    df_wide = (df_wide * list(range(0, shape[1]))).cumsum()
+    if not long:
+        return df_wide
+    else:
+        df_wide.index.name = "datetime"
+        df_long = df_wide.reset_index(drop=False).melt(
+            id_vars=['datetime'], var_name='series_id', value_name='value'
+        )
+        return df_long
+
+
+def load_sine(long=False, shape=None, start_date: str = "2021-01-01"):
+    """Create a dataset of just zeroes for testing edge case."""
+    if shape is None:
+        shape = (500, 5)
+    df_wide = pd.DataFrame(
+        np.ones(shape),
+        index=pd.date_range(start_date, periods=shape[0], freq="D"),
+        columns=range(shape[1]),
+    )
+    X = pd.to_numeric(df_wide.index, errors='coerce', downcast='integer').values
+
+    def sin_func(a, X):
+        return a * np.sin(1 * X) + a
+
+    for column in df_wide.columns:
+        df_wide[column] = sin_func(column, X)
+    if not long:
+        return df_wide
+    else:
+        df_wide.index.name = "datetime"
+        df_long = df_wide.reset_index(drop=False).melt(
+            id_vars=['datetime'], var_name='series_id', value_name='value'
+        )
+        return df_long
