@@ -1,6 +1,8 @@
 """
 Naives and Others Requiring No Additional Packages Beyond Numpy and Pandas
 """
+from math import ceil
+import warnings
 import random
 import datetime
 import numpy as np
@@ -46,7 +48,7 @@ class ZeroesNaive(ModelObject):
             verbose=verbose,
         )
 
-    def fit(self, df, future_regressor=[]):
+    def fit(self, df, future_regressor=None):
         """Train algorithm given data supplied
 
         Args:
@@ -57,7 +59,7 @@ class ZeroesNaive(ModelObject):
         return self
 
     def predict(
-        self, forecast_length: int, future_regressor=[], just_point_forecast=False
+        self, forecast_length: int, future_regressor=None, just_point_forecast=False
     ):
         """Generates forecast data immediately following dates of index supplied to .fit()
 
@@ -133,7 +135,7 @@ class LastValueNaive(ModelObject):
             random_seed=random_seed,
         )
 
-    def fit(self, df, future_regressor=[]):
+    def fit(self, df, future_regressor=None):
         """Train algorithm given data supplied
 
         Args:
@@ -149,7 +151,7 @@ class LastValueNaive(ModelObject):
         return self
 
     def predict(
-        self, forecast_length: int, future_regressor=[], just_point_forecast=False
+        self, forecast_length: int, future_regressor=None, just_point_forecast=False
     ):
         """Generates forecast data immediately following dates of index supplied to .fit()
 
@@ -231,7 +233,7 @@ class AverageValueNaive(ModelObject):
         )
         self.method = method
 
-    def fit(self, df, future_regressor=[]):
+    def fit(self, df, future_regressor=None):
         """Train algorithm given data supplied.
 
         Args:
@@ -265,7 +267,7 @@ class AverageValueNaive(ModelObject):
         return self
 
     def predict(
-        self, forecast_length: int, future_regressor=[], just_point_forecast=False
+        self, forecast_length: int, future_regressor=None, just_point_forecast=False
     ):
         """Generates forecast data immediately following dates of index supplied to .fit()
 
@@ -370,7 +372,7 @@ class SeasonalNaive(ModelObject):
                 self.lag_2 = 1
         self.method = method
 
-    def fit(self, df, future_regressor=[]):
+    def fit(self, df, future_regressor=None):
         """Train algorithm given data supplied.
 
         Args:
@@ -415,7 +417,7 @@ class SeasonalNaive(ModelObject):
     def predict(
         self,
         forecast_length: int,
-        future_regressor=[],
+        future_regressor=None,
         just_point_forecast: bool = False,
     ):
         """Generate forecast data immediately following dates of .fit().
@@ -560,7 +562,7 @@ class MotifSimulation(ModelObject):
         self.cutoff_minimum = cutoff_minimum
         self.point_method = point_method
 
-    def fit(self, df, future_regressor=[]):
+    def fit(self, df, future_regressor=None):
         """Train algorithm given data supplied.
 
         Args:
@@ -867,7 +869,7 @@ class MotifSimulation(ModelObject):
         return self
 
     def predict(
-        self, forecast_length: int, future_regressor=[], just_point_forecast=False
+        self, forecast_length: int, future_regressor=None, just_point_forecast=False
     ):
         """Generates forecast data immediately following dates of index supplied to .fit()
 
@@ -1145,7 +1147,7 @@ class Motif(ModelObject):
         self.max_windows = max_windows
         self.multivariate = multivariate
 
-    def fit(self, df, future_regressor=[]):
+    def fit(self, df, future_regressor=None):
         """Train algorithm given data supplied.
 
         Args:
@@ -1157,7 +1159,7 @@ class Motif(ModelObject):
         return self
 
     def predict(
-        self, forecast_length: int, future_regressor=[], just_point_forecast=False
+        self, forecast_length: int, future_regressor=None, just_point_forecast=False
     ):
         """Generates forecast data immediately following dates of index supplied to .fit()
 
@@ -1302,3 +1304,367 @@ class Motif(ModelObject):
             "k": self.k,
             "max_windows": self.max_windows,
         }
+
+
+def predict_reservoir(
+    df,
+    forecast_length,
+    prediction_interval=None,
+    warmup_pts=1,
+    k=2,
+    ridge_param=2.5e-6,
+    seed_pts: int = 1,
+    seed_weighted: str = None,
+):
+    """Nonlinear Variable Autoregression or 'Next-Generation Reservoir Computing'
+
+    based on https://github.com/quantinfo/ng-rc-paper-code/
+    Gauthier, D.J., Bollt, E., Griffith, A. et al. Next generation reservoir computing. Nat Commun 12, 5564 (2021).
+    https://doi.org/10.1038/s41467-021-25801-2
+    with adjustments to make it probabilistic
+
+    This is very slow and memory hungry when n series/dimensions gets big (ie > 50).
+    Already effectively parallelized by linpack it seems.
+    It's very sensitive to error in most recent data point!
+    The seed_pts and seed_weighted can help address that.
+
+    Args:
+        name (str): String to identify class
+        frequency (str): String alias of datetime index frequency or else 'infer'
+        prediction_interval (float): Confidence interval for probabilistic forecast
+        k (int): the AR order (keep this small, larger is slow and usually pointless)
+        ridge_param (float): standard lambda for ridge regression
+        warmup_pts (int): in reality, passing 1 here (no warmup) is fine
+        seed_pts (int): number of back steps to use to simulate future
+            if > 10, also increases search space of probabilistic forecast
+        seed_weighted (str): how to summarize most recent points if seed_pts > 1
+    """
+    assert k > 0, "nvar `k` must be > 0"
+    assert warmup_pts > 0, "nvar `warmup_pts` must be > 0"
+    assert df.shape[1] > k, "nvar input data must contain at least k+1 records"
+
+    n_pts = df.shape[1]
+    # handle short data edge case
+    min_train_pts = 12
+    max_warmup_pts = n_pts - min_train_pts
+    if warmup_pts >= max_warmup_pts:
+        warmup_pts = max_warmup_pts if max_warmup_pts > 0 else 1
+
+    traintime_pts = n_pts - warmup_pts  # round(traintime / dt)
+    warmtrain_pts = warmup_pts + traintime_pts
+    testtime_pts = forecast_length + 1  # round(testtime / dt)
+    maxtime_pts = n_pts  # round(maxtime / dt)
+
+    # input dimension
+    d = df.shape[0]
+    # size of the linear part of the feature vector
+    dlin = k * d
+    # size of nonlinear part of feature vector
+    dnonlin = int(dlin * (dlin + 1) / 2)
+    # total size of feature vector: constant + linear + nonlinear
+    dtot = 1 + dlin + dnonlin
+
+    # create an array to hold the linear part of the feature vector
+    x = np.zeros((dlin, maxtime_pts))
+
+    # fill in the linear part of the feature vector for all times
+    for delay in range(k):
+        for j in range(delay, maxtime_pts):
+            x[d * delay : d * (delay + 1), j] = df[:, j - delay]
+
+    # create an array to hold the full feature vector for training time
+    # (use ones so the constant term is already 1)
+    out_train = np.ones((dtot, traintime_pts))
+
+    # copy over the linear part (shift over by one to account for constant)
+    out_train[1 : dlin + 1, :] = x[:, warmup_pts - 1 : warmtrain_pts - 1]
+
+    # fill in the non-linear part
+    cnt = 0
+    for row in range(dlin):
+        for column in range(row, dlin):
+            # shift by one for constant
+            out_train[dlin + 1 + cnt] = (
+                x[row, warmup_pts - 1 : warmtrain_pts - 1]
+                * x[column, warmup_pts - 1 : warmtrain_pts - 1]
+            )
+            cnt += 1
+
+    # ridge regression: train W_out to map out_train to Lorenz[t] - Lorenz[t - 1]
+    W_out = (
+        (x[0:d, warmup_pts:warmtrain_pts] - x[0:d, warmup_pts - 1 : warmtrain_pts - 1])
+        @ out_train[:, :].T
+        @ np.linalg.pinv(
+            out_train[:, :] @ out_train[:, :].T + ridge_param * np.identity(dtot)
+        )
+    )
+
+    # create a place to store feature vectors for prediction
+    out_test = np.ones(dtot)  # full feature vector
+    x_test = np.zeros((dlin, testtime_pts))  # linear part
+
+    # copy over initial linear feature vector
+    x_test[:, 0] = x[:, warmtrain_pts - 1]
+
+    # do prediction
+    for j in range(testtime_pts - 1):
+        # copy linear part into whole feature vector
+        out_test[1 : dlin + 1] = x_test[:, j]  # shift by one for constant
+        # fill in the non-linear part
+        cnt = 0
+        for row in range(dlin):
+            for column in range(row, dlin):
+                # shift by one for constant
+                out_test[dlin + 1 + cnt] = x_test[row, j] * x_test[column, j]
+                cnt += 1
+        # fill in the delay taps of the next state
+        x_test[d:dlin, j + 1] = x_test[0 : (dlin - d), j]
+        # do a prediction
+        x_test[0:d, j + 1] = x_test[0:d, j] + W_out @ out_test[:]
+    pred = x_test[0:d, 1:]
+
+    if prediction_interval is not None or seed_pts > 1:
+        # this works by using n most recent points as different starting "seeds"
+        # this has the advantage of generating perfect bounds on perfect functions
+        # on real world data, well, things will be less perfect...
+        n_samples = 10 if seed_pts <= 10 else seed_pts
+        interval_list = []
+        for ns in range(n_samples):
+
+            out_test = np.ones(dtot)  # full feature vector
+            x_int = np.zeros((dlin, testtime_pts + n_samples))  # linear part
+            # copy over initial linear feature vector
+            x_int[:, 0] = x[:, warmtrain_pts - 2 - ns]
+            # do prediction
+            for j in range(testtime_pts - 1 + n_samples):
+                # copy linear part into whole feature vector
+                out_test[1 : dlin + 1] = x_int[:, j]  # shift by one for constant
+                # fill in the non-linear part
+                cnt = 0
+                for row in range(dlin):
+                    for column in range(row, dlin):
+                        # shift by one for constant
+                        out_test[dlin + 1 + cnt] = x_int[row, j] * x_int[column, j]
+                        cnt += 1
+                # fill in the delay taps of the next state
+                x_int[d:dlin, j + 1] = x_int[0 : (dlin - d), j]
+                # do a prediction
+                x_int[0:d, j + 1] = x_int[0:d, j] + W_out @ out_test[:]
+            start_slice = ns + 2
+            end_slice = start_slice + testtime_pts - 1
+            interval_list.append(x_int[:, start_slice:end_slice])
+
+        interval_list = np.array(interval_list)
+        if seed_pts > 1:
+            pred_int = np.concatenate(
+                [np.expand_dims(x_test[:, 1:], axis=0), interval_list]
+            )
+            # assuming interval_list has more recent first
+            if seed_weighted == "linear":
+                pred = np.average(
+                    pred_int, axis=0, weights=range(pred_int.shape[0], 0, -1)
+                )[0:d]
+            elif seed_weighted == "exponential":
+                pred = np.average(
+                    pred_int, axis=0, weights=np.geomspace(100, 1, pred_int.shape[0])
+                )[0:d]
+            else:
+                pred = np.quantile(pred_int, q=0.5, axis=0)[0:d]
+        pred_upper = np.nanquantile(interval_list, q=prediction_interval, axis=0)[0:d]
+        pred_upper = np.where(pred_upper < pred, pred, pred_upper)
+        pred_lower = np.nanquantile(interval_list, q=(1 - prediction_interval), axis=0)[
+            0:d
+        ]
+        pred_lower = np.where(pred_lower > pred, pred, pred_lower)
+        return pred, pred_upper, pred_lower
+    else:
+        return pred
+
+
+class NVAR(ModelObject):
+    """Nonlinear Variable Autoregression or 'Next-Generation Reservoir Computing'
+
+    based on https://github.com/quantinfo/ng-rc-paper-code/
+    Gauthier, D.J., Bollt, E., Griffith, A. et al. Next generation reservoir computing. Nat Commun 12, 5564 (2021).
+    https://doi.org/10.1038/s41467-021-25801-2
+    with adjustments to make it probabilistic and to scale better
+
+    Args:
+        name (str): String to identify class
+        frequency (str): String alias of datetime index frequency or else 'infer'
+        prediction_interval (float): Confidence interval for probabilistic forecast
+        k (int): the AR order (keep this small, larger is slow and usually pointless)
+        ridge_param (float): standard lambda for ridge regression
+        warmup_pts (int): in reality, passing 1 here (no warmup) is fine
+        batch_size (int): nvar scales exponentially, to scale linearly, series are split into batches of size n
+        batch_method (str): method for collecting series to make batches
+    """
+
+    def __init__(
+        self,
+        name: str = "NVAR",
+        frequency: str = 'infer',
+        prediction_interval: float = 0.9,
+        holiday_country: str = 'US',
+        random_seed: int = 2020,
+        verbose: int = 0,
+        k: int = 2,
+        ridge_param: float = 2.5e-6,
+        warmup_pts: int = 1,
+        seed_pts: int = 1,
+        seed_weighted: str = None,
+        batch_size: int = 5,
+        batch_method: str = "input_order",
+        **kwargs
+    ):
+        ModelObject.__init__(
+            self,
+            name,
+            frequency,
+            prediction_interval,
+            holiday_country=holiday_country,
+            random_seed=random_seed,
+            verbose=verbose,
+        )
+        self.k = k
+        self.ridge_param = ridge_param
+        self.warmup_pts = warmup_pts
+        self.seed_pts = seed_pts
+        self.seed_weighted = seed_weighted
+        self.batch_size = batch_size
+        self.batch_method = batch_method
+
+    def fit(self, df, future_regressor=None):
+        """Train algorithm given data supplied.
+
+        Args:
+            df (pandas.DataFrame): Datetime Indexed
+        """
+        self.basic_profile(df)
+        if self.batch_method == "med_sorted":
+            df = df.loc[:, df.median().sort_values(ascending=False).index]
+        elif self.batch_method == "std_sorted":
+            df = df.loc[:, df.std().sort_values(ascending=False).index]
+        self.new_col_names = df.columns
+        self.batch_steps = ceil(df.shape[1] / self.batch_size)
+        self.df_train = df.to_numpy().T
+        self.fit_runtime = datetime.datetime.now() - self.startTime
+        return self
+
+    def predict(
+        self, forecast_length: int, future_regressor=None, just_point_forecast=False
+    ):
+        """Generates forecast data immediately following dates of index supplied to .fit()
+
+        Args:
+            forecast_length (int): Number of periods of data to forecast ahead
+            regressor (numpy.Array): additional regressor, not used
+            just_point_forecast (bool): If True, return a pandas.DataFrame of just point forecasts
+
+        Returns:
+            Either a PredictionObject of forecasts and metadata, or
+            if just_point_forecast == True, a dataframe of point forecasts
+        """
+        predictStartTime = datetime.datetime.now()
+        the_index = self.create_forecast_index(forecast_length=forecast_length)
+        df_list, df_list_up, df_list_low = [], [], []
+        # since already uses 100% cpu, no point parallelizing
+        for stp in range(self.batch_steps):
+            srt = stp * self.batch_size
+            stop = srt + self.batch_size
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
+                fore, up, low = predict_reservoir(
+                    self.df_train[srt:stop],
+                    forecast_length=forecast_length,
+                    warmup_pts=self.warmup_pts,
+                    seed_pts=self.seed_pts,
+                    seed_weighted=self.seed_weighted,
+                    k=self.k,
+                    ridge_param=self.ridge_param,
+                    prediction_interval=self.prediction_interval,
+                )
+            df_list_up.append(
+                pd.DataFrame(
+                    up.T,
+                    columns=self.new_col_names[srt:stop],
+                    index=the_index,
+                )
+            )
+            df_list.append(
+                pd.DataFrame(
+                    fore.T,
+                    columns=self.new_col_names[srt:stop],
+                    index=the_index,
+                )
+            )
+            df_list_low.append(
+                pd.DataFrame(
+                    low.T,
+                    columns=self.new_col_names[srt:stop],
+                    index=the_index,
+                )
+            )
+        forecast = pd.concat(df_list, axis=1)[self.column_names]
+        if just_point_forecast:
+            return forecast
+        else:
+            upper_forecast = pd.concat(df_list_up, axis=1)[self.column_names]
+            lower_forecast = pd.concat(df_list_low, axis=1)[self.column_names]
+            predict_runtime = datetime.datetime.now() - predictStartTime
+            prediction = PredictionObject(
+                model_name=self.name,
+                forecast_length=forecast_length,
+                forecast_index=forecast.index,
+                forecast_columns=forecast.columns,
+                lower_forecast=lower_forecast,
+                forecast=forecast,
+                upper_forecast=upper_forecast,
+                prediction_interval=self.prediction_interval,
+                predict_runtime=predict_runtime,
+                fit_runtime=self.fit_runtime,
+                model_parameters=self.get_params(),
+            )
+            return prediction
+
+    def get_new_params(self, method: str = 'random'):
+        """Returns dict of new parameters for parameter tuning"""
+        k_choice = random.choices([1, 2, 3, 4, 5], [0.5, 0.5, 0.1, 0.001, 0.001])[0]
+        ridge_choice = random.choices(
+            [0, -1, -2, -3, -4, -5, -6, -7, -8],
+            [0.1, 0.1, 0.1, 0.5, 0.1, 0.1, 0.5, 0.1, 0.1],
+        )[0]
+        ridge_choice = 2 * 10 ** ridge_choice
+        warmup_pts_choice = random.choices([1, 50, 250], [0.9, 0.1, 0.1])[0]
+        seed_pts_choice = random.choices([1, 10, 100], [0.8, 0.2, 0.01])[0]
+        if seed_pts_choice > 1:
+            seed_weighted_choice = random.choices(
+                [None, "linear", "exponential"], [0.3, 0.3, 0.3]
+            )[0]
+        else:
+            seed_weighted_choice = None
+        batch_size_choice = random.choices([5, 10, 20, 30], [0.5, 0.2, 0.01, 0.001])[0]
+        batch_method_choice = random.choices(
+            ["input_order", "std_sorted", "max_sorted"], [0.5, 0.1, 0.1]
+        )[0]
+        return {
+            'k': k_choice,
+            'ridge_param': ridge_choice,
+            'warmup_pts': warmup_pts_choice,
+            'seed_pts': seed_pts_choice,
+            'seed_weighted': seed_weighted_choice,
+            'batch_size': batch_size_choice,
+            'batch_method': batch_method_choice,
+        }
+
+    def get_params(self):
+        """Return dict of current parameters."""
+        return {
+            'k': self.k,
+            'ridge_param': self.ridge_param,
+            'warmup_pts': self.warmup_pts,
+            'seed_pts': self.seed_pts,
+            'seed_weighted': self.seed_weighted,
+            'batch_size': self.batch_size,
+            'batch_method': self.batch_method,
+        }
