@@ -1,5 +1,6 @@
 """Preprocessing data methods."""
 import random
+import warnings
 import numpy as np
 import pandas as pd
 from autots.tools.impute import FillNA, df_interpolate
@@ -513,7 +514,11 @@ class SinTrend(EmptyTransformer):
         def sinfunc(t, A, w, p, c):
             return A * np.sin(w * t + p) + c
 
-        popt, pcov = scipy.optimize.curve_fit(sinfunc, tt, yy, p0=guess, maxfev=10000)
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            popt, pcov = scipy.optimize.curve_fit(
+                sinfunc, tt, yy, p0=guess, maxfev=10000
+            )
         A, w, p, c = popt
         # f = w/(2.*np.pi)
         # fitfunc = lambda t: A * np.sin(w*t + p) + c
@@ -537,16 +542,21 @@ class SinTrend(EmptyTransformer):
         X = pd.to_numeric(df.index, errors='coerce', downcast='integer').values
         self.sin_params = pd.DataFrame()
         # make this faster (250 columns in 2.5 seconds isn't bad, though)
+        fail_count = 0
         for column in df.columns:
+            vals = 0
             try:
                 y = df[column].values
                 vals = self.fit_sin(X, y)
                 current_param = pd.DataFrame(vals, index=[column])
             except Exception as e:
-                print(f"SinTrend failed with {repr(e)}")
+                print(f"SinTrend failed with {repr(e)} for {column} with {vals}")
                 current_param = pd.DataFrame(
                     {"amp": 0, "omega": 1, "phase": 1, "offset": 1}, index=[column]
                 )
+                fail_count += 1
+            if fail_count >= df.shape[1]:
+                raise ValueError("SinTrend Transformer failed on all series.")
             self.sin_params = pd.concat([self.sin_params, current_param], axis=0)
         self.shape = df.shape
         return self
@@ -1021,11 +1031,17 @@ class DatepartRegressionTransformer(EmptyTransformer):
         X = date_part(df.index, method=self.datepart_method)
         from autots.models.sklearn import retrieve_regressor
 
+        multioutput = True
+        if y.ndim < 2:
+            multioutput = False
+        elif y.shape[1] < 2:
+            multioutput = False
         self.model = retrieve_regressor(
             regression_model=self.regression_model,
             verbose=0,
             verbose_bool=False,
             random_seed=2020,
+            multioutput=multioutput,
         )
         self.model = self.model.fit(X, y)
         self.shape = df.shape
@@ -1375,21 +1391,17 @@ class Round(EmptyTransformer):
 
     def __init__(
         self,
-        method: str = "middle",
         decimals: int = 0,
         on_transform: bool = False,
         on_inverse: bool = True,
+        force_int: bool = False,
         **kwargs,
     ):
         super().__init__(name="Round")
-        self.method = method
-        self.decimals = decimals
+        self.decimals = int(decimals)
         self.on_transform = on_transform
         self.on_inverse = on_inverse
-
-        self.force_int = False
-        if decimals <= 0:
-            self.force_int = True
+        self.force_int = force_int
 
     @staticmethod
     def get_new_params(method: str = 'random'):
@@ -1399,7 +1411,6 @@ class Round(EmptyTransformer):
             on_inverse_c = True
         choice = random.choices([-2, -1, 0, 1, 2], [0.1, 0.2, 0.4, 0.2, 0.1], k=1)[0]
         return {
-            "model": "middle",
             "decimals": choice,
             "on_transform": on_transform_c,
             "on_inverse": on_inverse_c,
@@ -2408,8 +2419,8 @@ del superfast_transformer_dict['IntermittentOccurrence']
 del superfast_transformer_dict['cffilter']
 del superfast_transformer_dict['QuantileTransformer']
 del superfast_transformer_dict['PowerTransformer']
-del fast_transformer_dict['convolution_filter']
-del fast_transformer_dict['HPFilter']
+del superfast_transformer_dict['convolution_filter']
+del superfast_transformer_dict['HPFilter']
 
 # probability dictionary of FillNA methods
 na_probs = {
